%global pkg mlx4

%global kernel_version_min 4.18.0-315.el8

# To pick a specific kernel use `--define "kernel_version 4.18.0-315.el8"`
# Otherwise the latest installed kernel-devel version later than
# %%{kernel_version_min} will be used.

%global kernel_version_latest %({ echo %{kernel_version_min}; rpm --query kernel-devel --queryformat '%%{VERSION}-%%{RELEASE}\\\n' | grep -v "package kernel-devel is not installed"; } | sort -V | sed -e '0,/^%{kernel_version_min}$/d' | tail -n 1)

%if "%{kernel_version_latest}" != ""
%{!?kernel_version:%{expand:%%global kernel_version %{kernel_version_latest}}}
%endif

%global _use_internal_dependency_generator 0

%global debug_package %{nil}

%global __spec_install_post \
  %{?__debug_package:%{__debug_install_post}} \
  %{__arch_install_post} \
  %{__os_install_post} \
  %{__mod_compress_install_post}

%global __mod_compress_install_post find %{buildroot}/lib/modules -type f -name \*.ko -exec xz \{\} \\;


Name:             kmod-%{pkg}
Version:          4.18.0
Release:          326%{?dist}
Summary:          Mellanox ConnectX HCA (%{pkg}) driver

License:          GPLv2 and (BSD or GPLv2)
URL:              https://www.kernel.org/

Source0:          %{pkg}-%{kernel_version_min}.tar.xz
Patch1:           0001-disable-mlx4_en.patch
Patch2:           0002-enable-mlx4_core-gen2.patch

ExclusiveArch:    x86_64 aarch64

BuildRequires:    elfutils-libelf-devel
BuildRequires:    gcc
BuildRequires:    kernel-abi-whitelists
BuildRequires:    kernel-rpm-macros
BuildRequires:    kmod
BuildRequires:    make
BuildRequires:    redhat-rpm-config
BuildRequires:    xz

%{!?kernel_version:
BuildRequires:    kernel-devel >= %{kernel_version_min}
}

%{?kernel_version:
BuildRequires:    kernel-devel = %{kernel_version}

Supplements:      kernel = %{kernel_version}

Requires:         (kernel = %{kernel_version} if kernel)

Provides:         kernel-modules = %{kernel_version}.%{_arch}
}

Requires(post):   /usr/sbin/depmod
Requires(postun): /usr/sbin/depmod

Provides:         %{name} = %{?epoch:%{epoch}:}%{version}-%{release}


%description
This package provides the Mellanos ConnectX HCA (%{pkg}) driver. Compared to
the in-kernel driver this driver re-enables support for deprecated Mellanox
Gen2 and ConnectX-2 adapters:

- 0x15B3:0x1002: Mellanox ConnectX-2 MT25400
- 0x15B3:0x6340: Mellanox Hermon SDR
- 0x15B3:0x634A: Mellanox Hermon DDR
- 0x15B3:0x6354: Mellanox Hermon QDR
- 0x15B3:0x6368: Mellanox Hermon EN 10GigE
- 0x15B3:0x6372: Mellanox ConnectX EN 10GBASE-T
- 0x15B3:0x6732: Mellanox Hermon DDR Gen2
- 0x15B3:0x673C: Mellanox Hermon QDR Gen2
- 0x15B3:0x6746: Mellanox ConnectX EN 40GigE PCIe Gen2 5GT/s
- 0x15B3:0x6750: Mellanox Hermon EN 10GigE Gen2
- 0x15B3:0x675A: Mellanox ConnectX EN 10GBASE-T Gen2
- 0x15B3:0x6764: Mellanox ConnectX EN 10 GigE PCIe Gen2
- 0x15B3:0x676E: Mellanox ConnectX-2 40GigE PCIe Gen2


%prep
%autosetup -p1 -n %{pkg}-%{kernel_version_min}


%build
pushd src
%{__make} -C /usr/src/kernels/%{kernel_version}.%{_arch} %{?_smp_mflags} M=$PWD modules
popd


%install
%{__install} -D -t %{buildroot}/lib/modules/%{kernel_version}.%{_arch}/extra/drivers/net/ethernet/mellanox/%{pkg} src/%{pkg}_core.ko

# Make .ko objects temporarily executable for automatic stripping
find %{buildroot}/lib/modules -type f -name \*.ko -exec chmod u+x \{\} \+


%clean
%{__rm} -rf %{buildroot}


%post
/usr/sbin/depmod -aeF /lib/modules/%{kernel_version}.%{_arch}/System.map %{kernel_version}.%{_arch}


%postun
/usr/sbin/depmod -aeF /lib/modules/%{kernel_version}.%{_arch}/System.map %{kernel_version}.%{_arch}


%files
%defattr(644,root,root,755)
/lib/modules/%{kernel_version}.%{_arch}
%license COPYING


%changelog
* Thu Jul 29 2021 Peter Georg <peter.georg@physik.uni-regensburg.de> - 4.18.0-326
- Rebuild for 4.18.0-326.el8

* Fri Jul 23 2021 Peter Georg <peter.georg@physik.uni-regensburg.de> - 4.18-315
- Initial version
